@@ -13,8 +13,9 @@ bool cmWIXPatch::LoadFragments(std::string const& patchFilePath)
 {
   cmWIXPatchParser parser(Fragments, Logger);
   if (!parser.ParseFile(patchFilePath.c_str())) {
-    cmCPackLogger(cmCPackLog::LOG_ERROR, "Failed parsing XML patch file: '"
-                    << patchFilePath << "'" << std::endl);
+    cmCPackLogger(cmCPackLog::LOG_ERROR,
+                  "Failed parsing XML patch file: '" << patchFilePath << "'"
+                                                     << std::endl);
     return false;
   }
 
@@ -40,7 +41,7 @@ void cmWIXPatch::ApplyFragment(std::string const& id,
 void cmWIXPatch::ApplyElementChildren(const cmWIXPatchElement& element,
                                       cmWIXSourceWriter& writer)
 {
-  for (cmWIXPatchNode* node : element.children) {
+  for (const auto& node : element.children) {
     switch (node->type()) {
       case cmWIXPatchNode::ELEMENT:
         ApplyElement(dynamic_cast<const cmWIXPatchElement&>(*node), writer);
