@@ -1,7 +1,6 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
    file Copyright.txt or https://cmake.org/licensing for details.  */
-#ifndef cmAuxSourceDirectoryCommand_h
-#define cmAuxSourceDirectoryCommand_h
+#pragma once
 
 #include "cmConfigure.h" // IWYU pragma: keep
 
@@ -12,5 +11,3 @@ class cmExecutionStatus;
 
 bool cmAuxSourceDirectoryCommand(std::vector<std::string> const& args,
                                  cmExecutionStatus& status);
-
-#endif
