@@ -5,6 +5,7 @@
 #include "cmConfigure.h" // IWYU pragma: keep
 
 #include <map>
+#include <set>
 #include <string>
 #include <vector>
 
@@ -12,7 +13,6 @@
 
 class cmGeneratorTarget;
 class cmGlobalCommonGenerator;
-class cmLinkLineComputer;
 class cmLocalCommonGenerator;
 class cmMakefile;
 class cmSourceFile;
@@ -32,10 +32,6 @@ protected:
   // Feature query methods.
   cmValue GetFeature(const std::string& feature, const std::string& config);
 
-  // Helper to add flag for windows .def file.
-  void AddModuleDefinitionFlag(cmLinkLineComputer* linkLineComputer,
-                               std::string& flags, const std::string& config);
-
   cmGeneratorTarget* GeneratorTarget;
   cmMakefile* Makefile;
   cmLocalCommonGenerator* LocalCommonGenerator;
@@ -74,6 +70,9 @@ protected:
 
   std::string GetLinkerLauncher(const std::string& config);
 
+  bool HaveRequiredLanguages(const std::vector<cmSourceFile const*>& sources,
+                             std::set<std::string>& languagesNeeded) const;
+
 private:
   using ByLanguageMap = std::map<std::string, std::string>;
   struct ByConfig
