@@ -99,8 +99,6 @@ const char* cmGlobalVisualStudioGenerator::GetIDEVersion() const
   switch (this->Version) {
     case cmGlobalVisualStudioGenerator::VSVersion::VS9:
       return "9.0";
-    case cmGlobalVisualStudioGenerator::VSVersion::VS10:
-      return "10.0";
     case cmGlobalVisualStudioGenerator::VSVersion::VS11:
       return "11.0";
     case cmGlobalVisualStudioGenerator::VSVersion::VS12:
@@ -128,14 +126,6 @@ void cmGlobalVisualStudioGenerator::WriteSLNHeader(std::ostream& fout)
       fout << "Microsoft Visual Studio Solution File, Format Version 10.00\n";
       fout << "# Visual Studio 2008\n";
       break;
-    case cmGlobalVisualStudioGenerator::VSVersion::VS10:
-      fout << "Microsoft Visual Studio Solution File, Format Version 11.00\n";
-      if (this->ExpressEdition) {
-        fout << "# Visual C++ Express 2010\n";
-      } else {
-        fout << "# Visual Studio 2010\n";
-      }
-      break;
     case cmGlobalVisualStudioGenerator::VSVersion::VS11:
       fout << "Microsoft Visual Studio Solution File, Format Version 12.00\n";
       if (this->ExpressEdition) {
