@@ -1,17 +1,17 @@
 #!/usr/bin/env python
 
-import sys, os
+import sys
 
 if len(sys.argv) != 2:
   sys.exit(-1)
 name = sys.argv[1] + "/CMake.qhp"
 
-f = open(name)
+f = open(name, "rb")
 
 if not f:
   sys.exit(-1)
 
-lines = f.read().splitlines()
+lines = f.read().decode("utf-8").splitlines()
 
 if not lines:
   sys.exit(-1)
@@ -25,6 +25,8 @@ for line in lines:
              ("envvar", "envvar"),
              ("variable", "variable"),
              ("generator", "generator"),
+             ("genex", "genex"),
+             ("guide", "guide"),
              ("target property", "prop_tgt"),
              ("test property", "prop_test"),
              ("source file property", "prop_sf"),
@@ -37,7 +39,7 @@ for line in lines:
 
   for domain_object_string, domain_object_type in mapping:
     if "<keyword name=\"" + domain_object_string + "\"" in line:
-      if not "id=\"" in line and not "#index-" in line:
+      if "id=\"" not in line and "#index-" not in line:
         prefix = "<keyword name=\"" + domain_object_string + "\" "
         part1, part2 = line.split(prefix)
         head, tail = part2.split("#" + domain_object_type + ":")
@@ -45,5 +47,5 @@ for line in lines:
         line = part1 + prefix + "id=\"" + domain_object_type + "/" + domain_object + "\" " + part2
   newlines.append(line + "\n")
 
-f = open(name, "w")
-f.writelines(newlines)
+f = open(name, "wb")
+f.writelines(map(lambda line: line.encode("utf-8"), newlines))
